@@ -89,6 +89,7 @@ class hdimage(object):
         self.name = name
         self.section = 'KeyDevices\\passwords\\"{0}"'.format(self.name)
         self.user = user or getpass.getuser()
+        self.certs = {}
 
         self.configfile = '/var/opt/cprocsp/users/{0}/local.ini'.format(self.user)
         self.section = 'KeyDevices\\passwords\\"{0}"'.format(self.name)
@@ -96,6 +97,17 @@ class hdimage(object):
         self.broken = False
         try:
             self.ctx = csp.Crypt(self.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_SILENT)
+            key = self.ctx.get_key()
+            try:
+                cert = key.extract_cert(csp.AT_KEYEXCHANGE)
+                self.certs['AT_KEYEXCHANGE'] = _cert_info(cert)
+            except:
+                pass
+            try:
+                cert = key.extract_cert(csp.AT_SIGNATURE)
+                self.certs['AT_SIGNATURE'] = _cert_info(cert)
+            except:
+                pass
         except ValueError:
             try:
                 self.ctx = csp.Crypt(
@@ -227,27 +239,33 @@ class hdimage(object):
             open(os.path.join(res.dirname, fn), 'wb').write(data)
 
         del res.ctx
-        res.ctx = csp.Crypt(res.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_SILENT)
-        if paswd:
-            res.ctx.set_password(str(paswd), csp.AT_KEYEXCHANGE)
-            res.ctx.change_password(str(''))
-        for nsc in struct['non_standard']:
-            certdata = zf.read(nsc)
-            newc = csp.Cert(certdata)
-            newc.bind(res.ctx, csp.AT_KEYEXCHANGE)
-            cs = csp.CertStore(res.ctx, b"MY")
-            cs.add_cert(newc)
-            key = res.ctx.get_key(csp.AT_KEYEXCHANGE)
-            key.store_cert(newc)
-        for nsc in struct['standard']:
-            certdata = zf.read(nsc + '.cer')
-            newc = csp.Cert(certdata)
-            keyspec = getattr(csp, nsc)
-            newc.bind(res.ctx, keyspec)
-            cs = csp.CertStore(res.ctx, b"MY")
-            cs.add_cert(newc)
-            key = res.ctx.get_key(keyspec)
-            key.store_cert(newc)
+        try:
+            res.ctx = csp.Crypt(res.cont, csp.PROV_GOST_2001_DH, csp.CRYPT_SILENT)
+            if paswd:
+                res.ctx.set_password(str(paswd), csp.AT_KEYEXCHANGE)
+                res.ctx.change_password(str(''))
+            for nsc in struct['non_standard']:
+                certdata = zf.read(nsc)
+                res.certs['AT_KEYEXCHANGE'] = _cert_info(certdata)
+                newc = csp.Cert(certdata)
+                newc.bind(res.ctx, csp.AT_KEYEXCHANGE)
+                cs = csp.CertStore(res.ctx, b"MY")
+                cs.add_cert(newc)
+                key = res.ctx.get_key(csp.AT_KEYEXCHANGE)
+                key.store_cert(newc)
+            for nsc in struct['standard']:
+                certdata = zf.read(nsc + '.cer')
+                res.certs[nsc] = _cert_info(certdata)
+                newc = csp.Cert(certdata)
+                keyspec = getattr(csp, nsc)
+                newc.bind(res.ctx, keyspec)
+                cs = csp.CertStore(res.ctx, b"MY")
+                cs.add_cert(newc)
+                key = res.ctx.get_key(keyspec)
+                key.store_cert(newc)
+        except:
+            cryptoapi.remove_key(res.name)
+            raise
         return res
 
     @classmethod
